#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from logging.handlers import RotatingFileHandler
import time
import sys, time
import lbdaemon
from lbdaemon import Daemon
from lbconverter import convert_files
import config
import traceback
from multiprocessing import Pool
from lbrest import LBRest
import os
import shutil
import pkg_resources
from report import get_config_env



config.set_config()

# Set up log configurations
logger = logging.getLogger("LBConverter")
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# handler = logging.FileHandler(config.LOGFILE_PATH)
max_bytes = 1024*1024*20 # 20 MB
handler = RotatingFileHandler(config.LOGFILE_PATH, maxBytes=max_bytes, backupCount=10, encoding=None)
handler.setFormatter(formatter)
logger.addHandler(handler)

class LBConverter(Daemon):
    """ Light Base Extractor Daemon
    """

    def run(self):
        """ Overrided method used by super class.
            Crates a process pool object which controls a pool of worker processes.
            It supports asynchronous results with timeouts and callbacks and has a parallel map implementation.
            That means each base will be processed at same time.
        """
        lbrest = LBRest()
        
        while True:
            self.is_running = False
            self.has_started = True

            logger.info(u'## CONNECTING TO LBGENERATOR... ##')
            
            base_exists = lbrest.base_exists()
            if not base_exists:
                if not lbrest.create_base_log():
                    self.is_running = True
                    self.has_started = False
                    logger.error("Could not connect to server")

            if self.has_started:
                logger.info(u'## CONNECTION ESTABILISHED ##')
                self.clean_output_path()

                while not self.is_running:
                    bases = lbrest.get_bases()
                    if bases:
                        self.is_running = True
                        get_config_env()
                        try:
                            pool = Pool(processes=len(bases))
                            logger.info('## STARTING LBCONVERTER PROCESS --v%s ##' % config.VERSION)
                            pool.map(convert_files, bases)
                        except Exception as e:
                            logger.critical(str(e))
            else:
                logger.info(u"## WAITING FOR START CONDITIONS ##")
                # try again in 30 seconds
                time.sleep(30)

    def clean_output_path(self):
        shutil.rmtree(config.OUTPATH, ignore_errors=True)
        dirs = os.listdir('/tmp')
        for folder in dirs:
            if '-images' in folder:
                try:
                    shutil.rmtree('/tmp/'+folder)
                except OSError:
                    logger.error("Can not remove temporary folder")
                    continue

    def status(self):
        """
            Check process is running
        """
        # Check for a pidfile to see if the daemon already runs
        try:
            with open(self.pidfile, 'r') as pf:
                pid = int(pf.read().strip())
        except IOError:
            pid = None

        if pid is None:
            message = "pidfile {0} is not running. \n"
            sys.stdout.write(message.format(self.pidfile))
            sys.exit(0)
        elif self.check_pid(pid):
            message = "pidfile {0} is running. \n"
            sys.stdout.write(message.format(self.pidfile))
            sys.exit(0)
        else:
            message = "pidfile {0} is not running. \n"
            sys.stdout.write(message.format(self.pidfile))
            sys.exit(0)


if __name__ == "__main__":

    daemon = LBConverter(config.PIDFILE_PATH)

    if len(sys.argv) == 2:
        if 'start' == sys.argv[1]:
            sys.stdout.write("starting daemon ...\n")
            daemon.start()
        elif 'stop' == sys.argv[1]:
            sys.stdout.write("stopping daemon ...\n")
            daemon.stop()
        elif 'restart' == sys.argv[1]:
            sys.stdout.write("restarting daemon ...\n")
            daemon.restart()
        elif 'status' == sys.argv[1]:
            daemon.status()
        else:
            sys.stderr.write("Unknown command\n")
            sys.exit(1)
        sys.exit(0)
    else:
        sys.stderr.write("usage: %s start|stop|restart\n" % sys.argv[0])
        sys.exit(1)

