# -*- coding: utf-8 -*-

import subprocess
from subprocess import PIPE
import config
import glob
import os
import lbocr
import lbrtf
import logging
import lbxls

class DocumentConverter():
    """
    This class provides text extraction from doc, docx and odt files. docx and 
    odt files are zipped files. So unzip and sed command line are used. antiword and catdoc 
    command line is used for doc files. More info: 
    http://etienned.github.io/posts/extract-text-from-word-docx-simply/
    """

    _doc_exts = {
        'doc': None,
        'ppt': None,
        'pps': None,
        'docx': 'word/document.xml',
        'odt': 'content.xml',
        'odp': 'content.xml',
        'ods': 'content.xml',
        'rtf': 'rich_text',
        'pptx': 'ppt/slides/*.xml',
        'ppsx': 'ppt/slides/*.xml',
        'xlsx': 'spreadsheet',
        'xls': 'spreadsheet'
    }

    def __init__(self, file_path, file_name,file_format, id_file,catdoc_encoding='cp1252'):

        self.file_path = file_path
        self.file_name = file_name
        self.file_format = file_format
        self.id_file = id_file
        self.unzip_cmd = config.UNZIP_CMD
        self.sed_cmd = config.SED_CMD
        self.catdoc_cmd = config.CATDOC_CMD
        self.catppt_cmd = config.CATPPT_CMD
        self.catdoc_encoding = catdoc_encoding
        self.antiword_cmd = config.ANTIWORD_CMD
        self.antiword_resources = config.ANTIWORD_RESOURCES

        try:
            self._xml_file = self._doc_exts[self.file_format]
        except KeyError:
            raise KeyError('DocumentExtensioError: Format %s not supported' % self.file_format)

    def get_text(self):
        if 'rich_text' == self._xml_file:
            text = lbrtf.read_file(self.file_path, self.id_file)
        elif self.file_format in ['ppt','pps']:
            text = self._catppt()
        elif 'spreadsheet' == self._xml_file:
            text = lbxls.read_file(self.file_path)
            if self.file_format == 'xlsx':
                self._unzip()
                text = text + self.get_text_images(self.file_path,self.file_name)
        elif self._xml_file:
            unzip_ps = self._unzip()
            text = self._sed(unzip_ps)
            # logger.info("File name: %s\n File_path: %s\n", self.file_name, self.file_path)
            text = text + self.get_text_images(self.file_path,self.file_name)
            # logger.info("Got text images smoothly")
        else:
            text = self._antiword()
        return text

    def _unzip(self):
        
        """
            Tries first to get text from the .xml file and puts into unzip_output
        """
        if self._xml_file != 'spreadsheet':
            try:
                unzip_ps = subprocess.Popen(
                    [self.unzip_cmd, '-qc', self.file_path, self._xml_file],
                    stdout=PIPE, stderr=PIPE, stdin=PIPE)
            except OSError:
                raise OSError('OSError: unzip command not working properly')

            unzip_output, unzip_error= unzip_ps.communicate()
            if unzip_error:
                raise Exception('Error while unzip:' + str(unzip_error))
            text_output = unzip_output
        else:
            text_output = ''
        """
            Now, extracting the image folder: 
        """
        if self.file_format == 'docx':
            extract_subfolder = 'word/media/*'
        elif self.file_format in ['odt','ods','odp']:
            extract_subfolder = 'Pictures/*'
        elif self.file_format in ['pptx','ppsx']:
            extract_subfolder = 'ppt/media/image*'
        elif self.file_format == 'xlsx':
            extract_subfolder = 'xl/media/image*'
        else:
            extract_subfolder = ''
        try:
              unzip_ps = subprocess.Popen(
                [self.unzip_cmd,'-o', self.file_path,'-d',self.file_path +
                    '-extracted',extract_subfolder],
                 stdout=PIPE, stderr=PIPE, stdin=PIPE)
        except OSError:
            raise OSError('OSError: unzip command not working properly')
        unzip_output, unzip_error_images = unzip_ps.communicate()
        # logger.info(unzip_output) 
        if unzip_error_images:
            """
            There are no images in the document
            """
            pass
        return text_output
    def _sed(self, unzip_output):
        try:
            sed_ps = subprocess.Popen(
                [self.sed_cmd, r's#</w:p>#\n\n#g;s#<[^>]*>##g'],
                 stdout=PIPE, stderr=PIPE, stdin=PIPE)
        except OSError:
            raise OSError('OSError: sed command not working properly')

        sed_output, sed_error = sed_ps.communicate(unzip_output)

        if sed_error:
            raise Exception('Error while using sed: ' + str(sed_error))

        return sed_output

    def get_text_images(self, file_path, file_name):
        """
            Tries to extract text from the images
            Initially, only for .docx files

            TODO: Extraction for .odt and doc files
        """
        text_image = ""
        if self.file_format == 'docx':
            path_images  = file_path  +'-extracted/word/media/image'
        elif self.file_format in ['odt','ods','odp']:
            path_images  = file_path  +'-extracted/Pictures/'
        elif self.file_format in ['pptx','ppsx']:
            path_images = file_path + '-extracted/ppt/media/image'
        elif self.file_format == 'xlsx':
            path_images = file_path + '-extracted/xl/media/image'

        #logger.info("Trying to get text from images in folder %s... \n",path_images)
        #logger.info(sorted(glob.glob(path_images + '*')))    
        for image_path in sorted(glob.glob(path_images + '*')):
            try:
                #logger.info("Getting text from image: %s", image_path)
                text = lbocr.image_to_string(image_path).encode('utf-8')
            except lbocr.TesseractError as e:
                tesseract_error = True
                logger.warning('TesseractError while getting text from image: %s' % e)
            except Exception as e:
                tesseract_error = True
                logger.warning('Exception while getting text from image: %s' % e)
            else:
                image_label = '## '+ image_path.split('/')[-1].split('.')[0] + ' ##'
                text_image = text_image + image_label.encode('utf-8') + text
            finally:
                lbocr.cleanup(image_path)
                
        # logger.info("Tesseract ran softly.\n")
        return text_image

    def _catdoc(self):
        """ Fallback to catdoc """
        try:
            cmd = [self.catdoc_cmd, '-s', '8859-1','-d' ,'utf-8', self.file_path]
            catdoc_ps = subprocess.Popen(cmd,
                 stdout=PIPE, stderr=PIPE, stdin=PIPE)
        except OSError as e:
            raise OSError('OSError: catdoc command %s not working properly: %s' % (str(cmd), e))

        catdoc_output, catdoc_error = catdoc_ps.communicate()
        if catdoc_error:
            raise Exception('Error while using catdoc: ' + str(catdoc_error))
        
        return catdoc_output

    def _catppt(self):
              """ Used to get .ppt file text """
              try:
                  cmd = [self.catppt_cmd,'-d','utf-8', self.file_path]
                  catppt_ps = subprocess.Popen(cmd,
                          stdout=PIPE, stderr=PIPE, stdin=PIPE)
              except OSError as e:
                  raise OSError('OSError: catppt command %s not working properly: %s' % (str(cmd), e))
              catppt_output, catppt_error = catppt_ps.communicate()
              if catppt_error:
                  raise Exception('Error while using catppt: ' + str(catppt_error))
              return catppt_output

    def _antiword(self):
        """
            Used only with *.DOC files
            * If an RTF file, try to use lb_rtf.
            * In case of error attempts to use the catdoc.
        """
        try:
            cmd = [self.antiword_cmd, self.file_path]
            environ = dict(os.environ)
            environ['HOME'] = self.antiword_resources
            environ['LANG'] = 'pt_BR.UTF-8'
            antiword_ps = subprocess.Popen(cmd,
                                           stdout=PIPE, stderr=PIPE, stdin=PIPE, env=environ)
        except OSError as e:
            raise OSError('OSError: antiword command %s not working properly: %s' % (str(cmd), e))

        antiword_output, antiword_error = antiword_ps.communicate()
        if antiword_error:
            #raise Exception('Error while using antiword: ' + str(antiword_error))
            if 'Rich Text Format' in antiword_error:
                try:
                    antiword_output = lbrtf.read_file(self.file_path, self.id_file)
                except Exception, e:
                    raise Exception('Error while getting text from RTF file: ' + str(e))
            else:
                try:
                    antiword_output = self._catdoc()
                except Exception, e:
                    raise Exception('Error while using catdoc: ' + str(e))
        return antiword_output

logger = logging.getLogger("LBConverter")
