# -*- coding: utf-8 -*-

import codecs
import os 
import config
import subprocess 
import logging
import glob
import shutil

#Método antigo, substituído por pdf2image
def extract_pdf_images(pdf_path, out_path):
    """ @param pdf_path: pdf path on file system 
        @param out_path: file system path used to save images
        Return list of file system paths (images)
    """

    IMAGES = [ ]
    OUT_BASE = out_path + '/img' 
    CONVERT_EXT = '.bmp'

    if not os.path.exists(out_path):
        os.makedirs(out_path)

    command = [config.PDFIMAGES_CMD, pdf_path, OUT_BASE]
    proc = subprocess.Popen(command)
    (stdoutdata, stderrdata) = proc.communicate()

    for image_path in sorted(glob.glob(OUT_BASE + '*')):

        convert_out_path = image_path.split('.')[0] + CONVERT_EXT
        command = [config.CONVERT_CMD, image_path, convert_out_path] 
        proc = subprocess.Popen(command)
        (stdoutdata, stderrdata) = proc.communicate()
        cleanup(image_path)

        if os.path.exists(convert_out_path):
            IMAGES.append(convert_out_path)

    return IMAGES
#Método novo, que substitiu extract_pdf_images
def pdf2image(pdf_path, out_path):
    """ @param pdf_path: pdf path on file system 
        @param out_path: file system path used to save images
        Return list of file system paths (images)
    """
    #IMAGES = [ ]
    OUT_BASE = out_path + '/img' 

    if not os.path.exists(out_path):
        os.makedirs(out_path)

    command = [config.PDFIMAGES_CMD, pdf_path, OUT_BASE]
    proc = subprocess.Popen(command)
    (stdoutdata, stderrdata) = proc.communicate()

    return sorted(glob.glob(OUT_BASE + '*'))

def convert2bmp(image_path):
    """Convert imagen to bmp format
       @param image_path: path of image to convert
        Return path image converted.
    """
    CONVERT_EXT = '.bmp'
    convert_out_path = image_path.split('.')[0] + CONVERT_EXT
    if not image_path.endswith('.pict'):
        """
        Treating unsupported .pict image file
        """
        command = [config.CONVERT_CMD, image_path, '+matte', 'BMP3:'+convert_out_path] 
        proc = subprocess.Popen(command)
        (stdoutdata, stderrdata) = proc.communicate()
    cleanup(image_path)

    if os.path.exists(convert_out_path):
        return    convert_out_path
    else:
        return None

class TesseractError(Exception):
    """
    Exception raised when Tesseract fails.
    """
    def __init__(self, status, message):
        Exception.__init__(self, message)
        self.status = status
        self.message = message
        self.args = (status, message)

def image_to_string(image_path, lang=None, configs=None):
    '''
    Runs tesseract on the specified image. First, the image is written to disk,
    and then the tesseract command is run on the image. Tesseract's result is
    read, and the temporary files are erased.

    Arguments:
        image --- image to OCR
        lang --- tesseract language to use
        builder --- builder used to configure Tesseract and read its result.
            The builder is used to specify the type of output expected.
            Possible builders are TextBuilder or CharBoxBuilder. If builder ==
            None, the builder used will be TextBuilder.

    Returns:
        Depends of the specified builder. By default, it will return a simple
        string.
    '''
    input_file_name = convert2bmp(image_path)
    if input_file_name is None:
        return ""
    output_file_name_base = input_file_name
    # logger.info("Image converted: %s\n",output_file_name_base)
    (status, errors) = run_tesseract(input_file_name,
                                     output_file_name_base,
                                     lang=lang,
                                     configs=configs)
    if status:
        raise TesseractError(status, errors)

    output_file_name = ('%s.%s' % (output_file_name_base,'txt'))
    if not os.access(output_file_name, os.F_OK):
        raise TesseractError(1, 'File %s does not exist' % output_file_name)

    try:
        with codecs.open(output_file_name, 'r', encoding='utf-8',
                         errors='ignore') as file_desc:
            results = file_desc.read().strip()
        return results
    finally:
        cleanup(input_file_name)
        cleanup(output_file_name)

def cleanup(filename):
    ''' Tries to remove the given filename. Ignores non-existent files '''
    try:
        os.remove(filename)   
    except OSError:
        pass

def cleanup_dirs(path_images):
    """ Tries to remove directory. Ignores non-existent folder"""
    try:
        shutil.rmtree(path_images, ignore_errors=True)
    except OSError:
        pass

def run_tesseract(input_filename, output_filename_base, lang='por',
                  #configs=["-psm", '3']):
                  configs=None):
    '''
    Runs Tesseract:
        `TESSERACT_CMD` \
                `input_filename` \
                `output_filename_base` \
                [-l `lang`] \
                [`configs`]

    Arguments:
        input_filename --- image to read
        output_filename_base --- file name in which must be stored the result
            (without the extension)
        lang --- Tesseract language to use (if None, none will be specified)
        config --- List of Tesseract configs to use (if None, none will be
            specified)

    Returns:
        Returns (the exit status of Tesseract, Tesseract's output)
    '''

    command = [config.TESSERACT_CMD, input_filename, output_filename_base]

    if lang is not None:
        command += ['-l', lang]

    if configs != None:
        command += configs

    environ = dict(os.environ)

#    #logger.info("Using lang %s" % lang)
#    #logger.info("TESSDATA_PREFIX %s" % environ.get("TESSDATA_PREFIX", "Not Found") )
    environ['TESSDATA_PREFIX'] = config.TESSDATA_PREFIX
#    #logger.info("Using TESSDATA_PREFIX %s" % config.TESSDATA_PREFIX)
    proc = subprocess.Popen(command,
            #shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            env=environ)
    # Beware that in some cases, tesseract may print more on stderr than
    # allowed by the buffer of subprocess.Popen.stderr. So we must read stderr
    # asap or Tesseract will remain stuck when trying to write again on stderr.
    # In the end, we just have to make sure that proc.stderr.read() is called
    # before proc.wait()
    errors = proc.stdout.read()
    return (proc.wait(), errors)

logger = logging.getLogger("LBConverter")
